/*
* This file is part of fwmon
* Copyright (c) 2001 Gianni Tedesco
*
*    fwmon is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    fwmon is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with fwmon; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "netlink.h"
int netlink_fd=-1;

#ifdef NETLINK_USE_SOCKET
#include <asm/types.h>
#include <linux/netlink.h>
struct sockaddr_nl nls;
#endif

#ifndef NETLINK_USE_SOCKET

/*
 * Using a character device (depricated, i believe)
 */
int netlink_open(void)
{
	netlink_fd = open( NETLINK_DEV, O_RDONLY );
	if ( netlink_fd<0 ) {
		return 0;
	}

	return 1;
}

int netlink_read(void *data, int len)
{
	if ( netlink_fd<0 ){
		errno=-EBADF;
		return -EBADF;
	}
	return read( netlink_fd, data, len );
}

#else

/* 
 * Using a proper netlink socket
 */

int netlink_open(void)
{
	netlink_fd = socket( PF_NETLINK, SOCK_RAW, NETLINK_FIREWALL );
	if ( netlink_fd < 0 ) return 0;

	memset( &nls, 0, sizeof(nls));
	nls.nl_family = AF_NETLINK;
	nls.nl_groups = -1;

	if ( bind( netlink_fd, (struct sockaddr *)&nls, sizeof(nls)) ) return 0;

	return 1;
}

int netlink_read(void *data, int len)
{
	if ( netlink_fd<0 ) {
		errno=-EBADF;
		return -EBADF;
	}

	return recv( netlink_fd, data, len, 0 );
}

#endif

int netlink_close(void)
{
	return close(netlink_fd);
}
