/*
* This file is part of fwmon
* Copyright (c) 2001 Gianni Tedesco
*
*    fwmon is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    fwmon is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with fwmon; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "util.h"
char errbuf[64];
char *errstr=(char *)&errbuf;

char *strtoupper(char *in)
{
	char *tmp=in;
	
	/* AUDIT: As long as get_proto() is the only caller
	 * of this function, bounds checking is not needed, as
	 * get_proto() gives us data straight from getprotobynumber()
	 */
	do {
		*tmp = toupper(*tmp);
	}while ( *(tmp++) != '\0' );
	
	return in;
}

const char *get_tcpflags(struct tcphdr *tcp)
{
	static char buf[16];
	char *b=(char *)&buf;

	memset(b, 0, 16);
	
	strncat(b, "[", 1);
	if ( tcp->ack ) strncat(b, "A", 1);
	if ( tcp->psh ) strncat(b, "P", 1);
	if ( tcp->rst ) strncat(b, "R", 1);
	if ( tcp->syn ) strncat(b, "S", 1);
	if ( tcp->fin ) strncat(b, "F", 1);
	if ( tcp->urg ) strncat(b, "U", 1);
	strncat(b, "]", 1);

	return b;
}

const char *get_proto(int p)
{
	struct protoent *proto;

	switch (p) {
		case 1: return "ICMP";
		case 2: return "IGMP";
		case 6: return "TCP";
		case 17: return "UDP";
		case 41: return "IPv6";
		case 43: return "IPv6-route";
		case 44: return "IPv6-frag";
		case 50: return "IPv6-crypt";
		case 51: return "IPv6-auth";
		case 58: return "IPv6-icmp";
		case 59: return "IPv6-nonxt";
		case 60: return "IPv6-opts";
		case 94: return "IPIP";
		case 98: return "ENCAP";
	}
	
	if ( flag(FLAG_CHROOT) ) {
		sprintf(errstr, "(%i)", p);
		return errstr;
	}
	
	proto = getprotobynumber(p);

	/* AUDIT: Callers no longer need check for NULL */
	if (proto)
		return strtoupper(proto->p_name);
	else
		sprintf(errstr, "(%i)", p);

	return errstr;
}

int format_ip ( u_long ip, char *txt )
{
	struct in_addr addr;
	addr.s_addr=ip;
	strncpy( txt, inet_ntoa(addr), 127 );
	return 1;
}

/* this is disgusting */
char *icmp_type(u_char type, u_char code)
{
	switch ( type ) {
		case 0:
			return ("ICMP_ECHO_REPLY");
		case 3:
			switch ( code ){
				case 0:
					return ("ICMP_NET_UNREACH");
					
				case 1:
					return ("ICMP_HOST_UNREACH");
					
				case 2:
					return ("ICMP_PROTO_UNREACH");
					
				case 3:
					return ("ICMP_PORT_UNREACH");
					
				case 4:
					return ("ICMP_FRAG_NEEDED_BUT_DF_SET");
					
				case 5:
					return ("ICMP_SOURCE_ROUTE_FAILED");
					
				case 6:
					return ("ICMP_NET_UNKNOWN");
					
				case 7:
					return ("ICMP_HOST_UNKNOWN");
					
				case 8:
					return ("ICMP_SOURCE_HOST_ISOLATED");
					
				case 9:
					return ("ICMP_NET_ADMIN_PROHIBITED");
					
				case 10:
					return ("ICMP_HOST_ADMIN_PROHIBITED");
					
				case 11:
					return ("ICMP_NET_UNREACH_TOS");
					
				case 12:
					return ("ICMP_HOST_UNREACH_TOS");
					
				case 13:
					return ("ICMP_COMMS_FILTERED");
					
				case 14:
					return ("ICMP_PRECEDENCE_VIOLATION");
					
				case 15:
					return ("ICMP_PRECEDENCE_CUTOFF");
					
				default:
					return ("ICMP_DEST_UNCREACH");
					
			}
		case 4:
			return ("ICMP_SOURCE_QUENCH");
			
		case 5:
			switch ( code ){
				case 0:
					return ("ICMP_REDIRECT_NET");
					
				case 1:
					return ("ICMP_REDIRECT_HOST");
					
				case 2:
					return ("ICMP_REDIRECT_TOS_NET");
					
				case 3:
					return ("ICMP_REDIRECT_TOS_HOST");
					
				default:
					return ("ICMP_REDIRECT");
					
			}
		case 8:
			return ("ICMP_ECHO_REQUEST");
			
		case 9:
			return ("ICMP_ROUTER_ADVERTISMENT");
			
		case 10:
			return ("ICMP_ROUTER_SOLICITATION");
			
		case 11:
			if ( code == 0 ){
				return ("ICMP_TTL_EXCEEDED");
			}else if ( code == 1 ){
				return ("ICMP_TTL_EXCEEDED_IN_REASSEMBLY");
			}else{
				return ("ICMP_TIME_EXCEED");
			}
		case 12:
			if ( code == 0 ){
				return ("ICMP_IP_HEADER_BAD");
			}else if ( code == 1 ){
				return ("ICMP_IP_OPTION_MISSING");
			}else{
				return ("ICMP_BAD_PARAM");
			}
		case 13:
			return ("ICMP_TIMESTAMP_REQUEST");
			
		case 14:
			return ("ICMP_TIMESTAMP_REPLY");
			
		case 15:
			return ("ICMP_INFO_REQUEST");
			
		case 16:
			return ("ICMP_INFO_REPLY");
			
		case 17:
			return ("ICMP_MASK_REQUEST");
			
		case 18:
			return ("ICMP_MASK_REPLY");
			
	}

	sprintf(errstr, "ICMP_UNKNOWN (%.2x:%.2x)", type, code);
	return errstr;
}
